#
# Author: Seth Vargo <sethvargo@gmail.com>
#
# Copyright 2014 Chef Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'statsd'

module Falcore
  class Dumper::DogStatsd < Dumper::Base
    validate do
      presence!('DogStatsd host') { config.dogstatsd.host }
      presence!('DogStatsd port') { config.dogstatsd.port }
    end

    run do
      stat(master)
      master.slaves.each(&method(:stat))
    end

    private

    def statsd
      @statsd ||= Statsd.new(config.dogstatsd.host, config.dogstatsd.port)
    end

    #
    # @private
    #
    # Push stats to statsd for this particular +node+.
    #
    # @param [Node::Base] node
    #   the node to examine
    #
    def stat(node)
      options = {
        tags: ["name:#{node.display_name}"]
      }

      # Up/down
      statsd.gauge('jenkins.node.offline', node.offline? ? 1 : 0, options)

      # Idle
      statsd.gauge('jenkins.node.idle', node.idle? ? 1 : 0, options)

      # Response time
      statsd.timing('jenkins.node.response_time', node.response_time, options)

      # Temporary space
      statsd.gauge('jenkins.node.temporary_space', node.temporary_space, options)

      # Disk space
      statsd.gauge('jenkins.node.disk_space', node.disk_space, options)

      # Free memory
      statsd.gauge('jenkins.node.free_memory', node.free_memory, options)

      # Total memory
      statsd.gauge('jenkins.node.total_memory', node.total_memory, options)

      # Free swap
      statsd.gauge('jenkins.node.free_swap', node.free_swap, options)

      # Total swap
      statsd.gauge('jenkins.node.total_swap', node.total_swap, options)
    end
  end
end
