#
# Author: Seth Vargo <sethvargo@gmail.com>
#
# Copyright 2014 Chef Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

module Falcore
  class Node::Slave < Node::Base
    # @return [Node::Master]
    attr_reader :master

    #
    # @param [Node::Master] master
    # @param data (see Node::Base#initialize)
    #
    def initialize(master, data = {})
      super(data)

      unless master.is_a?(Node::Master)
        raise ArgumentError, "#{master.class} is not an Falcore::Node::Master!"
      end

      @master = master
      @master.add_slave(self)
    end

    #
    # Namespace the slave under it's master.
    #
    # @return (see Base#id)
    #
    def id
      "#{@master.id}.#{display_name.gsub(/\./, '-')}"
    end
  end
end
